@@ -16,7 +16,6 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-from __future__ import print_function
 """Joining a domain."""
 
 from samba.auth import system_session
@@ -41,7 +40,7 @@ from samba import werror
 from base64 import b64encode
 from samba import WERRORError, NTSTATUSError
 from samba import sd_utils
-from samba.dnsserver import ARecord, AAAARecord, CNameRecord
+from samba.dnsserver import ARecord, AAAARecord, CNAMERecord
 import logging
 import random
 import time
@@ -49,8 +48,7 @@ import re
 import os
 import tempfile
 from collections import OrderedDict
-from samba.compat import text_type
-from samba.compat import get_string
+from samba.common import get_string
 from samba.netcmd import CommandError
 
 
@@ -137,7 +135,7 @@ class DCJoinContext(object):
         if machinepass is not None:
             ctx.acct_pass = machinepass
         else:
-            ctx.acct_pass = samba.generate_random_machine_password(128, 255)
+            ctx.acct_pass = samba.generate_random_machine_password(120, 120)
 
         ctx.dnsdomain = ctx.samdb.domain_dns_name()
 
@@ -258,8 +256,9 @@ class DCJoinContext(object):
 
         ctx.del_noerror(res[0].dn, recursive=True)
 
-        if "msDS-Krbtgtlink" in res[0]:
-            new_krbtgt_dn = res[0]["msDS-Krbtgtlink"][0]
+        krbtgt_dn = res[0].get('msDS-KrbTgtLink', idx=0)
+        if krbtgt_dn is not None:
+            ctx.new_krbtgt_dn = krbtgt_dn
             ctx.del_noerror(ctx.new_krbtgt_dn)
 
         res = ctx.samdb.search(base=ctx.samdb.get_default_basedn(),
@@ -338,7 +337,7 @@ class DCJoinContext(object):
                                attrs=["msDS-krbTgtLink", "userAccountControl", "serverReferenceBL", "rIDSetReferences"])
         if len(res) == 0:
             raise Exception("Could not find domain member account '%s' to promote to a DC, use 'samba-tool domain join' instead'" % ctx.samname)
-        if "msDS-krbTgtLink" in res[0] or "serverReferenceBL" in res[0] or "rIDSetReferences" in res[0]:
+        if "msDS-KrbTgtLink" in res[0] or "serverReferenceBL" in res[0] or "rIDSetReferences" in res[0]:
             raise Exception("Account '%s' appears to be an active DC, use 'samba-tool domain join' if you must re-create this account" % ctx.samname)
         if (int(res[0]["userAccountControl"][0]) & (samba.dsdb.UF_WORKSTATION_TRUST_ACCOUNT |
                                                     samba.dsdb.UF_SERVER_TRUST_ACCOUNT) == 0):
@@ -400,20 +399,6 @@ class DCJoinContext(object):
                                 ldb.OID_COMPARATOR_AND, samba.dsdb.SYSTEM_FLAG_CR_NTDS_DOMAIN))
         return str(res[0].dn)
 
-    def get_naming_master(ctx):
-        '''get the parent domain partition DN from parent DNS name'''
-        res = ctx.samdb.search(base='CN=Partitions,%s' % ctx.config_dn, attrs=['fSMORoleOwner'],
-                               scope=ldb.SCOPE_BASE, controls=["extended_dn:1:1"])
-        if 'fSMORoleOwner' not in res[0]:
-            raise DCJoinException("Can't find naming master on partition DN %s in %s" % (ctx.partition_dn, ctx.samdb.url))
-        try:
-            master_guid = str(misc.GUID(ldb.Dn(ctx.samdb, res[0]['fSMORoleOwner'][0].decode('utf8')).get_extended_component('GUID')))
-        except KeyError:
-            raise DCJoinException("Can't find GUID in naming master on partition DN %s" % res[0]['fSMORoleOwner'][0])
-
-        master_host = '%s._msdcs.%s' % (master_guid, ctx.dnsforest)
-        return master_host
-
     def get_mysid(ctx):
         '''get the SID of the connected user. Only works with w2k8 and later,
            so only used for RODC join'''
@@ -505,7 +490,7 @@ class DCJoinContext(object):
                     v = [rec[a]]
                 else:
                     v = rec[a]
-                v = [x.encode('utf8') if isinstance(x, text_type) else x for x in v]
+                v = [x.encode('utf8') if isinstance(x, str) else x for x in v]
                 rattr = ctx.tmp_samdb.dsdb_DsReplicaAttribute(ctx.tmp_samdb, a, v)
                 attrs.append(rattr)
 
@@ -746,7 +731,7 @@ class DCJoinContext(object):
             except ldb.LdbError as e2:
                 (num, _) = e2.args
                 if num != ldb.ERR_UNWILLING_TO_PERFORM:
-                    pass
+                    raise
                 ctx.net.set_password(account_name=ctx.samname,
                                      domain_name=ctx.domain_name,
                                      newpassword=ctx.acct_pass)
@@ -933,13 +918,13 @@ class DCJoinContext(object):
 
         secrets_ldb = Ldb(ctx.paths.secrets, session_info=system_session(), lp=ctx.lp)
 
-        presult = provision_fill(ctx.local_samdb, secrets_ldb,
-                                 ctx.logger, ctx.names, ctx.paths,
-                                 dom_for_fun_level=DS_DOMAIN_FUNCTION_2003,
-                                 targetdir=ctx.targetdir, samdb_fill=FILL_SUBDOMAIN,
-                                 machinepass=ctx.acct_pass, serverrole="active directory domain controller",
-                                 lp=ctx.lp, hostip=ctx.names.hostip, hostip6=ctx.names.hostip6,
-                                 dns_backend=ctx.dns_backend, adminpass=ctx.adminpass)
+        provision_fill(ctx.local_samdb, secrets_ldb,
+                       ctx.logger, ctx.names, ctx.paths,
+                       dom_for_fun_level=DS_DOMAIN_FUNCTION_2003,
+                       targetdir=ctx.targetdir, samdb_fill=FILL_SUBDOMAIN,
+                       machinepass=ctx.acct_pass, serverrole="active directory domain controller",
+                       lp=ctx.lp, hostip=ctx.names.hostip, hostip6=ctx.names.hostip6,
+                       dns_backend=ctx.dns_backend, adminpass=ctx.adminpass)
         print("Provision OK for domain %s" % ctx.names.dnsdomain)
 
     def create_replicator(ctx, repl_creds, binding_options):
@@ -951,7 +936,7 @@ class DCJoinContext(object):
     def join_replicate(ctx):
         """Replicate the SAM."""
 
-        print("Starting replication")
+        ctx.logger.info("Starting replication")
         ctx.local_samdb.transaction_start()
         try:
             source_dsa_invocation_id = misc.GUID(ctx.samdb.get_invocation_id())
@@ -1031,12 +1016,13 @@ class DCJoinContext(object):
             ctx.source_dsa_invocation_id = source_dsa_invocation_id
             ctx.destination_dsa_guid = destination_dsa_guid
 
-            print("Committing SAM database")
+            ctx.logger.info("Committing SAM database - this may take some time")
         except:
             ctx.local_samdb.transaction_cancel()
             raise
         else:
             ctx.local_samdb.transaction_commit()
+            ctx.logger.info("Committed SAM database")
 
         # A large replication may have caused our LDB connection to the
         # remote DC to timeout, so check the connection is still alive
@@ -1145,7 +1131,6 @@ class DCJoinContext(object):
         except WERRORError as e:
             if e.args[0] == werror.WERR_DNS_ERROR_NAME_DOES_NOT_EXIST:
                 name_found = False
-                pass
 
         if name_found:
             for rec in res.rec:
@@ -1207,7 +1192,7 @@ class DCJoinContext(object):
                             % (msdcs_cname, msdcs_zone, cname_target))
 
             add_rec_buf = dnsserver.DNS_RPC_RECORD_BUF()
-            rec = CNameRecord(cname_target)
+            rec = CNAMERecord(cname_target)
             add_rec_buf.rec = rec
             dns_conn.DnssrvUpdateRecord2(client_version,
                                          0,
@@ -1581,62 +1566,6 @@ def join_clone(logger=None, server=None, creds=None, lp=None,
     return ctx
 
 
-def join_subdomain(logger=None, server=None, creds=None, lp=None, site=None,
-                   netbios_name=None, targetdir=None, parent_domain=None, dnsdomain=None,
-                   netbios_domain=None, machinepass=None, adminpass=None, use_ntvfs=False,
-                   dns_backend=None, plaintext_secrets=False,
-                   backend_store=None, backend_store_size=None):
-    """Join as a DC."""
-    ctx = DCJoinContext(logger, server, creds, lp, site, netbios_name,
-                        targetdir, parent_domain, machinepass, use_ntvfs,
-                        dns_backend, plaintext_secrets,
-                        backend_store=backend_store,
-                        backend_store_size=backend_store_size)
-    ctx.subdomain = True
-    if adminpass is None:
-        ctx.adminpass = samba.generate_random_password(12, 32)
-    else:
-        ctx.adminpass = adminpass
-    ctx.parent_domain_name = ctx.domain_name
-    ctx.domain_name = netbios_domain
-    ctx.realm = dnsdomain
-    ctx.parent_dnsdomain = ctx.dnsdomain
-    ctx.parent_partition_dn = ctx.get_parent_partition_dn()
-    ctx.dnsdomain = dnsdomain
-    ctx.partition_dn = "CN=%s,CN=Partitions,%s" % (ctx.domain_name, ctx.config_dn)
-    ctx.naming_master = ctx.get_naming_master()
-    if ctx.naming_master != ctx.server:
-        logger.info("Reconnecting to naming master %s" % ctx.naming_master)
-        ctx.server = ctx.naming_master
-        ctx.samdb = SamDB(url="ldap://%s" % ctx.server,
-                          session_info=system_session(),
-                          credentials=ctx.creds, lp=ctx.lp)
-        res = ctx.samdb.search(base="", scope=ldb.SCOPE_BASE, attrs=['dnsHostName'],
-                               controls=[])
-        ctx.server = res[0]["dnsHostName"]
-        logger.info("DNS name of new naming master is %s" % ctx.server)
-
-    ctx.base_dn = samba.dn_from_dns_name(dnsdomain)
-    ctx.forestsid = ctx.domsid
-    ctx.domsid = security.random_sid()
-    ctx.acct_dn = None
-    ctx.dnshostname = "%s.%s" % (ctx.myname.lower(), ctx.dnsdomain)
-    # Windows uses 240 bytes as UTF16 so we do
-    ctx.trustdom_pass = samba.generate_random_machine_password(120, 120)
-
-    ctx.userAccountControl = samba.dsdb.UF_SERVER_TRUST_ACCOUNT | samba.dsdb.UF_TRUSTED_FOR_DELEGATION
-
-    ctx.SPNs.append('E3514235-4B06-11D1-AB04-00C04FC2DCD2/$NTDSGUID/%s' % ctx.dnsdomain)
-    ctx.secure_channel_type = misc.SEC_CHAN_BDC
-
-    ctx.replica_flags |= (drsuapi.DRSUAPI_DRS_WRIT_REP |
-                          drsuapi.DRSUAPI_DRS_FULL_SYNC_IN_PROGRESS)
-    ctx.domain_replica_flags = ctx.replica_flags
-
-    ctx.do_join()
-    ctx.logger.info("Created domain %s (SID %s) as a DC" % (ctx.domain_name, ctx.domsid))
-
-
 class DCCloneContext(DCJoinContext):
     """Clones a remote DC."""
 
